@@ -422,18 +422,21 @@ gs_job_died (GSJob *job)
         if (job->priv->pid > 0) {
                 int exit_status;
 
+                gs_debug ("Waiting on process %d", job->priv->pid);
                 exit_status = wait_on_child (job->priv->pid);
 
                 job->priv->status = GS_JOB_DEAD;
 
                 if (WIFEXITED (exit_status) && (WEXITSTATUS (exit_status) != 0)) {
+                        gs_debug ("Wait on child process failed");
                 } else {
                         /* exited normally */
                 }
         }
-
         g_spawn_close_pid (job->priv->pid);
         job->priv->pid = 0;
+
+        gs_debug ("Job died");
 }
 
 static void
@@ -723,7 +726,6 @@ command_watch (GIOChannel   *source,
 
         if (done) {
                 gs_job_died (job);
-                gs_debug ("job died");
 
                 job->priv->watch_id = 0;
                 return FALSE;
@@ -753,6 +755,8 @@ gs_job_start (GSJob *job)
         g_return_val_if_fail (job != NULL, FALSE);
         g_return_val_if_fail (GS_IS_JOB (job), FALSE);
 
+        gs_debug ("starting job");
+
         if (job->priv->pid != 0) {
                 g_warning ("Cannot restart active job.");
                 return FALSE;
@@ -807,7 +811,10 @@ gs_job_stop (GSJob *job)
         g_return_val_if_fail (job != NULL, FALSE);
         g_return_val_if_fail (GS_IS_JOB (job), FALSE);
 
+        gs_debug ("stopping job");
+
         if (job->priv->pid == 0) {
+                gs_debug ("Could not stop job: pid not defined");
                 return FALSE;
         }
 
@@ -833,6 +840,8 @@ gs_job_suspend (GSJob   *job,
         g_return_val_if_fail (job != NULL, FALSE);
         g_return_val_if_fail (GS_IS_JOB (job), FALSE);
 
+        gs_debug ("suspending job");
+
         if (job->priv->pid == 0) {
                 return FALSE;
         }
