@@ -28,6 +28,7 @@
 #include <errno.h>
 #include <time.h>
 #include <unistd.h>
+#include <sys/utsname.h>
 
 #include <glib/gprintf.h>
 #include <glib/gstdio.h>
@@ -44,67 +45,14 @@
 
 #include "gs-lock-plug.h"
 
-#include "passwd.h"
+#include "gs-debug.h"
 
 #include "fusa-manager.h"
 
-/* Profiling stuff adapted from gtkfilechooserdefault */
-
 #define KEY_LOCK_DIALOG_THEME "/apps/gnome-screensaver/lock_dialog_theme"
+#define KEY_TRY_AUTH_FIRST    "/apps/gnome-screensaver/try_auth_first"
 
-#undef PROFILE_LOCK_DIALOG
-#ifdef PROFILE_LOCK_DIALOG
-
-#define PROFILE_INDENT 4
-static int profile_indent;
-
-static void
-profile_add_indent (int indent)
-{
-        profile_indent += indent;
-        if (profile_indent < 0) {
-                g_error ("You screwed up your indentation");
-        }
-}
-
-static void
-_gs_lock_plug_profile_log (const char *func,
-                           int         indent,
-                           const char *msg1,
-                           const char *msg2)
-{
-        char *str;
-
-        if (indent < 0) {
-                profile_add_indent (indent);
-        }
-
-        if (profile_indent == 0) {
-                str = g_strdup_printf ("MARK: %s: %s %s %s", G_STRLOC, func, msg1 ? msg1 : "", msg2 ? msg2 : "");
-        } else {
-                str = g_strdup_printf ("MARK: %s: %*c %s %s %s", G_STRLOC, profile_indent - 1, ' ', func, msg1 ? msg1 : "", msg2 ? msg2 : "");
-        }
-
-        access (str, F_OK);
-
-        g_free (str);
-
-        if (indent > 0) {
-                profile_add_indent (indent);
-        }
-}
-
-#define profile_start(x, y) _gs_lock_plug_profile_log (G_STRFUNC, PROFILE_INDENT, x, y)
-#define profile_end(x, y)   _gs_lock_plug_profile_log (G_STRFUNC, -PROFILE_INDENT, x, y)
-#define profile_msg(x, y)   _gs_lock_plug_profile_log (NULL, 0, x, y)
-#else
-#define profile_start(x, y)
-#define profile_end(x, y)
-#define profile_msg(x, y)
-#endif
-
-
-enum { 
+enum {
         AUTH_PAGE = 0,
         SWITCH_PAGE
 };
@@ -112,12 +60,8 @@ enum {
 #define FACE_ICON_SIZE 48
 #define DIALOG_TIMEOUT_MSEC 60000
 
-static void gs_lock_plug_class_init (GSLockPlugClass *klass);
-static void gs_lock_plug_init       (GSLockPlug      *plug);
 static void gs_lock_plug_finalize   (GObject         *object);
 
-static gboolean password_check_idle_cb (GSLockPlug *plug);
-
 #define GS_LOCK_PLUG_GET_PRIVATE(o) (G_TYPE_INSTANCE_GET_PRIVATE ((o), GS_TYPE_LOCK_PLUG, GSLockPlugPrivate))
 
 struct GSLockPlugPrivate
@@ -130,10 +74,11 @@ struct GSLockPlugPrivate
         GtkWidget   *auth_face_image;
         GtkWidget   *auth_realname_label;
         GtkWidget   *auth_username_label;
-        GtkWidget   *auth_short_username_label;
-        GtkWidget   *auth_password_entry;
+        GtkWidget   *auth_prompt_label;
+        GtkWidget   *auth_prompt_entry;
+        GtkWidget   *auth_prompt_box;
         GtkWidget   *auth_capslock_label;
-        GtkWidget   *auth_status_label;
+        GtkWidget   *auth_message_label;
         GtkWidget   *switch_user_treeview;
 
         GtkWidget   *auth_unlock_button;
@@ -153,8 +98,7 @@ struct GSLockPlugPrivate
         guint        timeout;
 
         guint        idle_id;
-        guint        password_check_idle_id;
-        guint        password_reset_idle_id;
+        guint        auth_check_idle_id;
         guint        response_idle_id;
 
         GTimeVal     start_time;
@@ -181,8 +125,7 @@ enum {
         PROP_SWITCH_ENABLED
 };
 
-static GObjectClass *parent_class = NULL;
-static guint         lock_plug_signals [LAST_SIGNAL];
+static guint lock_plug_signals [LAST_SIGNAL];
 
 G_DEFINE_TYPE (GSLockPlug, gs_lock_plug, GTK_TYPE_PLUG)
 
@@ -192,8 +135,8 @@ gs_lock_plug_style_set (GtkWidget *widget,
 {
         GSLockPlug *plug;
 
-        if (GTK_WIDGET_CLASS (parent_class)->style_set) {
-                GTK_WIDGET_CLASS (parent_class)->style_set (widget, previous_style);
+        if (GTK_WIDGET_CLASS (gs_lock_plug_parent_class)->style_set) {
+                GTK_WIDGET_CLASS (gs_lock_plug_parent_class)->style_set (widget, previous_style);
         }
 
         plug = GS_LOCK_PLUG (widget);
@@ -289,16 +232,18 @@ static void
 set_status_text (GSLockPlug *plug,
                  const char *text)
 {
-        if (plug->priv->auth_status_label != NULL) {
-                gtk_label_set_text (GTK_LABEL (plug->priv->auth_status_label), text);
+        if (plug->priv->auth_message_label != NULL) {
+                gtk_label_set_text (GTK_LABEL (plug->priv->auth_message_label), text);
         }
 }
 
-static void
-set_dialog_sensitive (GSLockPlug *plug,
-                      gboolean    sensitive)
+void
+gs_lock_plug_set_sensitive (GSLockPlug *plug,
+                            gboolean    sensitive)
 {
-        gtk_widget_set_sensitive (plug->priv->auth_password_entry, sensitive);
+        g_return_if_fail (GS_IS_LOCK_PLUG (plug));
+
+        gtk_widget_set_sensitive (plug->priv->auth_prompt_entry, sensitive);
         gtk_widget_set_sensitive (plug->priv->auth_action_area, sensitive);
 }
 
@@ -311,24 +256,6 @@ remove_monitor_idle (GSLockPlug *plug)
         }
 }
 
-static void
-remove_password_check_idle (GSLockPlug *plug)
-{
-        if (plug->priv->password_check_idle_id > 0) {
-                g_source_remove (plug->priv->password_check_idle_id);
-                plug->priv->password_check_idle_id = 0;
-        }
-}
-
-static void
-remove_password_reset_idle (GSLockPlug *plug)
-{
-        if (plug->priv->password_reset_idle_id > 0) {
-                g_source_remove (plug->priv->password_reset_idle_id);
-                plug->priv->password_reset_idle_id = 0;
-        }
-}
-
 static void
 remove_response_idle (GSLockPlug *plug)
 {
@@ -355,12 +282,10 @@ gs_lock_plug_response (GSLockPlug *plug,
         }
 
         remove_monitor_idle (plug);
-        remove_password_check_idle (plug);
-        remove_password_reset_idle (plug);
         remove_response_idle (plug);
 
         if (response_id == GS_LOCK_PLUG_RESPONSE_CANCEL) {
-                gtk_entry_set_text (GTK_ENTRY (plug->priv->auth_password_entry), "");
+                gtk_entry_set_text (GTK_ENTRY (plug->priv->auth_prompt_entry), "");
         }
 
         if (response_id == GS_LOCK_PLUG_RESPONSE_OK) {
@@ -370,14 +295,7 @@ gs_lock_plug_response (GSLockPlug *plug,
                         current_page = gtk_notebook_get_current_page (GTK_NOTEBOOK (plug->priv->notebook));
                 }
 
-                if (current_page == AUTH_PAGE) {
-                        set_dialog_sensitive (plug, FALSE);
-                        set_status_text (plug, _("Checking password..."));
-
-                        plug->priv->password_check_idle_id = g_idle_add ((GSourceFunc)password_check_idle_cb,
-                                                                         plug);
-                        return;
-                } else {
+                if (current_page != AUTH_PAGE) {
                         /* switch user */
                         switch_user_response (plug);
                         return;
@@ -391,7 +309,7 @@ gs_lock_plug_response (GSLockPlug *plug,
 }
 
 static gboolean
-response_idle_cb (GSLockPlug *plug)
+response_cancel_idle_cb (GSLockPlug *plug)
 {
         plug->priv->response_idle_id = 0;
 
@@ -414,7 +332,7 @@ monitor_progress (GSLockPlug *plug)
         remaining = plug->priv->timeout - elapsed;
 
         if ((remaining <= 0) || (remaining > plug->priv->timeout)) {
-                set_dialog_sensitive (plug, FALSE);
+                gs_lock_plug_set_sensitive (plug, FALSE);
                 set_status_text (plug, _("Time has expired."));
 
                 if (plug->priv->response_idle_id != 0) {
@@ -424,7 +342,7 @@ monitor_progress (GSLockPlug *plug)
                 remove_response_idle (plug);
 
                 plug->priv->response_idle_id = g_timeout_add (2000,
-                                                              (GSourceFunc)response_idle_cb,
+                                                              (GSourceFunc)response_cancel_idle_cb,
                                                               plug);
                 return FALSE;
         }
@@ -479,30 +397,187 @@ restart_monitor_progress (GSLockPlug *plug)
                                              plug);
 }
 
+void
+gs_lock_plug_get_text (GSLockPlug *plug,
+                       char      **text)
+{
+        const char *typed_text;
+        char       *null_text;
+        char       *local_text;
+
+        typed_text = gtk_entry_get_text (GTK_ENTRY (plug->priv->auth_prompt_entry));
+        local_text = g_locale_from_utf8 (typed_text, strlen (typed_text), NULL, NULL, NULL);
+
+        null_text = g_strnfill (strlen (typed_text) + 1, '\b');
+        gtk_entry_set_text (GTK_ENTRY (plug->priv->auth_prompt_entry), null_text);
+        gtk_entry_set_text (GTK_ENTRY (plug->priv->auth_prompt_entry), "");
+        g_free (null_text);
+
+        if (text != NULL) {
+                *text = local_text;
+        }
+}
+
+typedef struct
+{
+        GSLockPlug *plug;
+        gint response_id;
+        GMainLoop *loop;
+        gboolean destroyed;
+} RunInfo;
+
+static void
+shutdown_loop (RunInfo *ri)
+{
+        if (g_main_loop_is_running (ri->loop))
+                g_main_loop_quit (ri->loop);
+}
+
+static void
+run_unmap_handler (GSLockPlug *plug,
+                   gpointer data)
+{
+        RunInfo *ri = data;
+
+        shutdown_loop (ri);
+}
+
+static void
+run_response_handler (GSLockPlug *plug,
+                      gint response_id,
+                      gpointer data)
+{
+        RunInfo *ri;
+
+        ri = data;
+
+        ri->response_id = response_id;
+
+        shutdown_loop (ri);
+}
+
+static gint
+run_delete_handler (GSLockPlug *plug,
+                    GdkEventAny *event,
+                    gpointer data)
+{
+        RunInfo *ri = data;
+
+        shutdown_loop (ri);
+
+        return TRUE; /* Do not destroy */
+}
+
+static void
+run_destroy_handler (GSLockPlug *plug,
+                     gpointer data)
+{
+        RunInfo *ri = data;
+
+        /* shutdown_loop will be called by run_unmap_handler */
+        ri->destroyed = TRUE;
+}
+
+/* adapted from GTK+ gtkdialog.c */
+int
+gs_lock_plug_run (GSLockPlug *plug)
+{
+        RunInfo ri = { NULL, GTK_RESPONSE_NONE, NULL, FALSE };
+        gboolean was_modal;
+        gulong response_handler;
+        gulong unmap_handler;
+        gulong destroy_handler;
+        gulong delete_handler;
+
+        g_return_val_if_fail (GS_IS_LOCK_PLUG (plug), -1);
+
+        g_object_ref (plug);
+
+        was_modal = GTK_WINDOW (plug)->modal;
+        if (!was_modal) {
+                gtk_window_set_modal (GTK_WINDOW (plug), TRUE);
+        }
+
+        if (!GTK_WIDGET_VISIBLE (plug)) {
+                gtk_widget_show (GTK_WIDGET (plug));
+        }
+
+        response_handler =
+                g_signal_connect (plug,
+                                  "response",
+                                  G_CALLBACK (run_response_handler),
+                                  &ri);
+
+        unmap_handler =
+                g_signal_connect (plug,
+                                  "unmap",
+                                  G_CALLBACK (run_unmap_handler),
+                                  &ri);
+
+        delete_handler =
+                g_signal_connect (plug,
+                                  "delete_event",
+                                  G_CALLBACK (run_delete_handler),
+                                  &ri);
+
+        destroy_handler =
+                g_signal_connect (plug,
+                                  "destroy",
+                                  G_CALLBACK (run_destroy_handler),
+                                  &ri);
+
+        ri.loop = g_main_loop_new (NULL, FALSE);
+
+        GDK_THREADS_LEAVE ();
+        g_main_loop_run (ri.loop);
+        GDK_THREADS_ENTER ();
+
+        g_main_loop_unref (ri.loop);
+
+        ri.loop = NULL;
+
+        if (!ri.destroyed) {
+                if (! was_modal) {
+                        gtk_window_set_modal (GTK_WINDOW (plug), FALSE);
+                }
+
+                g_signal_handler_disconnect (plug, response_handler);
+                g_signal_handler_disconnect (plug, unmap_handler);
+                g_signal_handler_disconnect (plug, delete_handler);
+                g_signal_handler_disconnect (plug, destroy_handler);
+        }
+
+        g_object_unref (plug);
+
+        return ri.response_id;
+}
+
 static void
 gs_lock_plug_show (GtkWidget *widget)
 {
-        profile_start ("start", NULL);
+        GSLockPlug *plug = GS_LOCK_PLUG (widget);
 
-        profile_start ("start", "parent");
-        if (GTK_WIDGET_CLASS (parent_class)->show) {
-                GTK_WIDGET_CLASS (parent_class)->show (widget);
+        gs_profile_start (NULL);
+
+        gs_profile_start ("parent");
+        if (GTK_WIDGET_CLASS (gs_lock_plug_parent_class)->show) {
+                GTK_WIDGET_CLASS (gs_lock_plug_parent_class)->show (widget);
         }
 
-        profile_end ("end", "parent");
+        gs_profile_end ("parent");
 
-        capslock_update (GS_LOCK_PLUG (widget), is_capslock_on ());
+        capslock_update (plug, is_capslock_on ());
 
-        restart_monitor_progress (GS_LOCK_PLUG (widget));
+        restart_monitor_progress (plug);
 
-        profile_end ("end", NULL);
+        gs_profile_end (NULL);
 }
 
 static void
 gs_lock_plug_hide (GtkWidget *widget)
 {
-        if (GTK_WIDGET_CLASS (parent_class)->hide) {
-                GTK_WIDGET_CLASS (parent_class)->hide (widget);
+        if (GTK_WIDGET_CLASS (gs_lock_plug_parent_class)->hide) {
+                GTK_WIDGET_CLASS (gs_lock_plug_parent_class)->hide (widget);
         }
 }
 
@@ -515,8 +590,8 @@ gs_lock_plug_size_request (GtkWidget      *widget,
         int mod_width;
         int mod_height;
 
-        if (GTK_WIDGET_CLASS (parent_class)->size_request) {
-                GTK_WIDGET_CLASS (parent_class)->size_request (widget, requisition);
+        if (GTK_WIDGET_CLASS (gs_lock_plug_parent_class)->size_request) {
+                GTK_WIDGET_CLASS (gs_lock_plug_parent_class)->size_request (widget, requisition);
         }
 
         /* don't constrain size when themed */
@@ -650,15 +725,14 @@ static void
 gs_lock_plug_close (GSLockPlug *plug)
 {
         /* Synthesize delete_event to close dialog. */
-  
+
         GtkWidget *widget = GTK_WIDGET (plug);
         GdkEvent  *event;
 
         event = gdk_event_new (GDK_DELETE);
-  
         event->any.window = g_object_ref (widget->window);
         event->any.send_event = TRUE;
-  
+
         gtk_main_do_event (event);
         gdk_event_free (event);
 }
@@ -670,8 +744,6 @@ gs_lock_plug_class_init (GSLockPlugClass *klass)
         GtkWidgetClass *widget_class = GTK_WIDGET_CLASS (klass);
         GtkBindingSet  *binding_set;
 
-        parent_class = g_type_class_peek_parent (klass);
-
         object_class->finalize     = gs_lock_plug_finalize;
         object_class->get_property = gs_lock_plug_get_property;
         object_class->set_property = gs_lock_plug_set_property;
@@ -724,66 +796,11 @@ gs_lock_plug_class_init (GSLockPlugClass *klass)
                                                                G_PARAM_READWRITE));
 
         binding_set = gtk_binding_set_by_class (klass);
-  
+
         gtk_binding_entry_add_signal (binding_set, GDK_Escape, 0,
                                       "close", 0);
 }
 
-static gboolean
-password_reset_idle_cb (GSLockPlug *plug)
-{
-        gtk_entry_set_text (GTK_ENTRY (plug->priv->auth_password_entry), "");
-        set_status_text (plug, "");
-        set_dialog_sensitive (plug, TRUE);
-
-        restart_monitor_progress (plug);
-
-        gtk_widget_grab_focus (plug->priv->auth_password_entry);
-
-        return FALSE;
-}
-
-static gboolean
-password_check_idle_cb (GSLockPlug *plug)
-{
-        const char *typed_password;
-        char       *null_password;
-        char       *local_password;
-
-        plug->priv->password_check_idle_id = 0;
-
-        typed_password = gtk_entry_get_text (GTK_ENTRY (plug->priv->auth_password_entry));
-        local_password = g_locale_from_utf8 (typed_password, strlen (typed_password), NULL, NULL, NULL);
-
-        null_password = g_strnfill (strlen (typed_password) + 1, '\b');
-        gtk_entry_set_text (GTK_ENTRY (plug->priv->auth_password_entry), null_password);
-        gtk_entry_set_text (GTK_ENTRY (plug->priv->auth_password_entry), "");
-        g_free (null_password);
-
-        if (validate_password (local_password, FALSE)) {
-                g_signal_emit (plug,
-                               lock_plug_signals [RESPONSE],
-                               0,
-                               GS_LOCK_PLUG_RESPONSE_OK);
-        } else {
-                remove_password_reset_idle (plug);
-
-                plug->priv->password_reset_idle_id = g_timeout_add (3000,
-                                                                    (GSourceFunc)password_reset_idle_cb,
-                                                                    plug);
-
-                set_status_text (plug, _("That password was incorrect."));
-
-                printf ("NOTICE=AUTH FAILED\n");
-                fflush (stdout);
-        }
-
-        memset (local_password, '\b', strlen (local_password));
-        g_free (local_password);
-
-        return FALSE;
-}
-
 static void
 switch_page (GSLockPlug *plug,
              GtkButton  *button)
@@ -839,6 +856,31 @@ logout_button_clicked (GtkButton  *button,
         }
 }
 
+void
+gs_lock_plug_show_prompt (GSLockPlug *plug,
+                          const char *message,
+                          gboolean    visible)
+{
+        g_return_if_fail (GS_IS_LOCK_PLUG (plug));
+
+        gs_debug ("Setting prompt to: %s", message);
+
+        gtk_label_set_text (GTK_LABEL (plug->priv->auth_prompt_label), message);
+        gtk_entry_set_visibility (GTK_ENTRY (plug->priv->auth_prompt_entry), visible);
+        gtk_widget_grab_focus (plug->priv->auth_prompt_entry);
+
+        restart_monitor_progress (plug);
+}
+
+void
+gs_lock_plug_show_message (GSLockPlug *plug,
+                           const char *message)
+{
+        g_return_if_fail (GS_IS_LOCK_PLUG (plug));
+
+        set_status_text (plug, message ? message : "");
+}
+
 /* button press handler used to inhibit popup menu */
 static gint
 entry_button_press (GtkWidget      *widget,
@@ -904,7 +946,7 @@ get_user_display_label (FusaUser *user)
 {
         char *label;
 
-        label = g_strdup_printf ("<big>%s</big>\n<small>%s</small>", 
+        label = g_strdup_printf ("<big>%s</big>\n<small>%s</small>",
                                  fusa_user_get_display_name (user),
                                  fusa_user_get_user_name (user));
 
@@ -953,34 +995,34 @@ populate_model (GSLockPlug   *plug,
         int           icon_size = FACE_ICON_SIZE;
         GtkIconTheme *theme;
 
-        profile_start ("start", NULL);
+        gs_profile_start (NULL);
 
         if (gtk_widget_has_screen (plug->priv->switch_user_treeview)) {
                 theme = gtk_icon_theme_get_for_screen (gtk_widget_get_screen (plug->priv->switch_user_treeview));
         } else {
                 theme = gtk_icon_theme_get_default ();
         }
-        
+
         pixbuf = gtk_icon_theme_load_icon (theme, "gdm", icon_size, 0, NULL);
 
-        profile_start ("start", "FUSA list users");
+        gs_profile_start ("FUSA list users");
         if (! plug->priv->fusa_manager) {
                 if (! g_thread_supported ()) {
-                        profile_start ("start", "g_thread_init");
+                        gs_profile_start ("g_thread_init");
                         g_thread_init (NULL);
-                        profile_end ("end", "g_thread_init");
+                        gs_profile_end ("g_thread_init");
                 }
-                profile_start ("start", "gnome_vfs_init");
+                gs_profile_start ("gnome_vfs_init");
                 gnome_vfs_init ();
-                profile_end ("end", "gnome_vfs_init");
+                gs_profile_end ("gnome_vfs_init");
 
-                profile_start ("start", "fusa_manager_ref_default");
+                gs_profile_start ("fusa_manager_ref_default");
                 plug->priv->fusa_manager = fusa_manager_ref_default ();
-                profile_end ("end", "fusa_manager_ref_default");
+                gs_profile_end ("fusa_manager_ref_default");
         }
 
         users = fusa_manager_list_users (plug->priv->fusa_manager);
-        profile_end ("end", "FUSA list users");
+        gs_profile_end ("FUSA list users");
 
         while (users) {
                 FusaUser           *user;
@@ -1026,7 +1068,7 @@ populate_model (GSLockPlug   *plug,
                 users = g_slist_delete_link (users, users);
         }
 
-        profile_end ("end", NULL);
+        gs_profile_end (NULL);
 }
 
 static int
@@ -1088,7 +1130,7 @@ separator_func (GtkTreeModel *model,
         gboolean is_separator;
 
         gtk_tree_model_get (model, iter, column, &text, -1);
-        
+
         if (text && strcmp (text, "__separator") == 0) {
                 is_separator = TRUE;
         } else {
@@ -1132,6 +1174,36 @@ filter_out_users (GtkTreeModel *model,
         return visible;
 }
 
+static gboolean
+row_activated_cb (GtkTreeView       *view,
+                  GtkTreePath       *path,
+                  GtkTreeViewColumn *column,
+                  GSLockPlug        *plug)
+{
+        GtkTreeModel *filter_model;
+        GtkTreeModel *model;
+        GtkTreeIter   iter;
+        GtkTreePath  *child_path;
+        char         *username;
+
+        filter_model = gtk_tree_view_get_model (view);
+        model = gtk_tree_model_filter_get_model (GTK_TREE_MODEL_FILTER (filter_model));
+
+        child_path = gtk_tree_model_filter_convert_path_to_child_path (GTK_TREE_MODEL_FILTER (filter_model), path);
+
+        gtk_tree_model_get_iter (model, &iter, child_path);
+        gtk_tree_model_get (model, &iter, USER_NAME_COLUMN, &username, -1);
+
+        /* FIXME: do something with username? */
+
+        switch_user_response (plug);
+
+        g_free (username);
+        gtk_tree_path_free (child_path);
+
+        return TRUE;
+}
+
 static void
 setup_treeview (GSLockPlug *plug)
 {
@@ -1145,7 +1217,7 @@ setup_treeview (GSLockPlug *plug)
                 return;
         }
 
-        profile_start ("start", NULL);
+        gs_profile_start (NULL);
 
         store = gtk_list_store_new (N_COLUMNS,
                                     G_TYPE_STRING,
@@ -1193,7 +1265,12 @@ setup_treeview (GSLockPlug *plug)
         gtk_tree_sortable_set_sort_column_id (GTK_TREE_SORTABLE (store),
                                               0,
                                               GTK_SORT_ASCENDING);
-        profile_end ("end", NULL);
+
+        g_signal_connect (GTK_TREE_VIEW (plug->priv->switch_user_treeview),
+                          "row-activated",
+                          G_CALLBACK (row_activated_cb), plug);
+
+        gs_profile_end (NULL);
 }
 
 static gboolean
@@ -1225,31 +1302,6 @@ get_user_display_name (void)
         return utf8_name;
 }
 
-static char *
-get_user_on_host_name (void)
-{
-        const char *name;
-        const char *host_name;
-        char       *utf8_name;
-        char       *str;
-
-        name = g_get_user_name ();
-        host_name = g_get_host_name ();
-
-        utf8_name = NULL;
-        if (name != NULL) {
-                utf8_name = g_locale_to_utf8 (name, -1, NULL, NULL, NULL);
-        }
-
-        /* Translators: this is "username on hostname".  You
-           can translate this to "%s@%s" if it is otherwise tricky
-           for your language. */
-        str = g_strdup_printf (_("%s on %s"), utf8_name, host_name);
-        g_free (utf8_name);
-
-        return str;
-}
-
 static char *
 get_user_name (void)
 {
@@ -1352,7 +1404,7 @@ static void
 create_page_one_buttons (GSLockPlug *plug)
 {
 
-        profile_start ("start", "page one buttons");
+        gs_profile_start ("page one buttons");
 
         plug->priv->auth_switch_button =  gs_lock_plug_add_button (GS_LOCK_PLUG (plug),
                                                                    plug->priv->auth_action_area,
@@ -1379,13 +1431,13 @@ create_page_one_buttons (GSLockPlug *plug)
 
         gtk_window_set_default (GTK_WINDOW (plug), plug->priv->auth_unlock_button);
 
-        profile_end ("end", "page one buttons");
+        gs_profile_end ("page one buttons");
 }
 
 static void
 create_page_two_buttons (GSLockPlug *plug)
 {
-        profile_start ("start", "page two buttons");
+        gs_profile_start ("page two buttons");
         plug->priv->switch_cancel_button =  gs_lock_plug_add_button (GS_LOCK_PLUG (plug),
                                                                      plug->priv->switch_action_area,
                                                                      GTK_STOCK_CANCEL);
@@ -1395,7 +1447,131 @@ create_page_two_buttons (GSLockPlug *plug)
                                                                      plug->priv->switch_action_area,
                                                                      _("S_witch User..."));
 
-        profile_end ("end", "page two buttons");
+        gs_profile_end ("page two buttons");
+}
+
+/* adapted from GDM */
+static char *
+expand_string (const char *text)
+{
+        GString       *str;
+        const char    *p;
+        char          *username;
+        int            i;
+        int            n_chars;
+        struct utsname name;
+
+        str = g_string_sized_new (strlen (text));
+
+        p = text;
+        n_chars = g_utf8_strlen (text, -1);
+        i = 0;
+
+        while (i < n_chars) {
+                gunichar ch;
+
+                ch = g_utf8_get_char (p);
+
+                /* Backslash commands */
+                if (ch == '\\') {
+                        p = g_utf8_next_char (p);
+                        i++;
+                        ch = g_utf8_get_char (p);
+
+                        if (i >= n_chars || ch == '\0') {
+                                g_warning ("Unescaped \\ at end of text\n");
+                                goto bail;
+                        } else if (ch == 'n') {
+                                g_string_append_unichar (str, '\n');
+                        } else {
+                                g_string_append_unichar (str, ch);
+                        }
+                } else if (ch == '%') {
+                        p = g_utf8_next_char (p);
+                        i++;
+                        ch = g_utf8_get_char (p);
+
+                        if (i >= n_chars || ch == '\0') {
+                                g_warning ("Unescaped %% at end of text\n");
+                                goto bail;
+                        }
+
+                        switch (ch) {
+                        case '%':
+                                g_string_append (str, "%");
+                                break;
+                        case 'c':
+                                /* clock */
+                                break;
+                        case 'd':
+                                /* display */
+                                g_string_append (str, g_getenv ("DISPLAY"));
+                                break;
+                        case 'h':
+                                /* hostname */
+                                g_string_append (str, g_get_host_name ());
+                                break;
+                        case 'm':
+                                /* machine name */
+                                uname (&name);
+                                g_string_append (str, name.machine);
+                                break;
+                        case 'n':
+                                /* nodename */
+                                uname (&name);
+                                g_string_append (str, name.nodename);
+                                break;
+                        case 'r':
+                                /* release */
+                                uname (&name);
+                                g_string_append (str, name.release);
+                                break;
+                        case 'R':
+                                /* Real name */
+                                username = get_user_display_name ();
+                                g_string_append (str, username);
+                                g_free (username);
+                                break;
+                        case 's':
+                                /* system name */
+                                uname (&name);
+                                g_string_append (str, name.sysname);
+                                break;
+                        case 'U':
+                                /* Username */
+                                username = get_user_name ();
+                                g_string_append (str, username);
+                                g_free (username);
+                                break;
+                        default:
+                                if (ch < 127) {
+                                        g_warning ("unknown escape code %%%c in text\n", (char)ch);
+                                } else {
+                                        g_warning ("unknown escape code %%(U%x) in text\n", (int)ch);
+                                }
+                        }
+                } else {
+                        g_string_append_unichar (str, ch);
+                }
+                p = g_utf8_next_char (p);
+                i++;
+        }
+
+ bail:
+
+        return g_string_free (str, FALSE);
+}
+
+static void
+expand_string_for_label (GtkWidget *label)
+{
+        const char *template;
+        char       *str;
+
+        template = gtk_label_get_label (GTK_LABEL (label));
+        str = expand_string (template);
+        gtk_label_set_label (GTK_LABEL (label), str);
+        g_free (str);
 }
 
 static void
@@ -1407,9 +1583,8 @@ create_page_one (GSLockPlug *plug)
         GtkWidget            *vbox2;
         GtkWidget            *hbox;
         char                 *str;
-        char                 *name;
 
-        profile_start ("start", "page one");
+        gs_profile_start ("page one");
 
         align = gtk_alignment_new (0.5, 0.5, 1, 1);
         gtk_notebook_append_page (GTK_NOTEBOOK (plug->priv->notebook), align, NULL);
@@ -1424,20 +1599,18 @@ create_page_one (GSLockPlug *plug)
         vbox2 = gtk_vbox_new (FALSE, 0);
         gtk_box_pack_start (GTK_BOX (vbox), vbox2, FALSE, FALSE, 0);
 
-        name = get_user_display_name ();
-        str = g_strdup_printf ("<span size=\"x-large\">%s</span>", name);
-        g_free (name);
+        str = g_strdup ("<span size=\"x-large\">%R</span>");
         plug->priv->auth_realname_label = gtk_label_new (str);
         g_free (str);
+        expand_string_for_label (plug->priv->auth_realname_label);
         gtk_misc_set_alignment (GTK_MISC (plug->priv->auth_realname_label), 0.5, 0.5);
         gtk_label_set_use_markup (GTK_LABEL (plug->priv->auth_realname_label), TRUE);
         gtk_box_pack_start (GTK_BOX (vbox2), plug->priv->auth_realname_label, FALSE, FALSE, 0);
 
-        name = get_user_on_host_name ();
-        str = g_strdup_printf ("<span size=\"small\">%s</span>", name);
-        g_free (name);
+        str = g_strdup_printf ("<span size=\"small\">%s</span>", _("%U on %h"));
         plug->priv->auth_username_label = gtk_label_new (str);
         g_free (str);
+        expand_string_for_label (plug->priv->auth_username_label);
         gtk_misc_set_alignment (GTK_MISC (plug->priv->auth_username_label), 0.5, 0.5);
         gtk_label_set_use_markup (GTK_LABEL (plug->priv->auth_username_label), TRUE);
         gtk_box_pack_start (GTK_BOX (vbox2), plug->priv->auth_username_label, FALSE, FALSE, 0);
@@ -1452,11 +1625,11 @@ create_page_one (GSLockPlug *plug)
         gtk_misc_set_alignment (GTK_MISC (password_label), 0, 0.5);
         gtk_box_pack_start (GTK_BOX (hbox), password_label, FALSE, FALSE, 0);
 
-        plug->priv->auth_password_entry = gtk_entry_new ();
-        gtk_box_pack_start (GTK_BOX (hbox), plug->priv->auth_password_entry, TRUE, TRUE, 0);
+        plug->priv->auth_prompt_entry = gtk_entry_new ();
+        gtk_box_pack_start (GTK_BOX (hbox), plug->priv->auth_prompt_entry, TRUE, TRUE, 0);
 
         gtk_label_set_mnemonic_widget (GTK_LABEL (password_label),
-                                       plug->priv->auth_password_entry);
+                                       plug->priv->auth_prompt_entry);
 
         plug->priv->auth_capslock_label = gtk_label_new ("");
         gtk_misc_set_alignment (GTK_MISC (plug->priv->auth_capslock_label), 0.5, 0.5);
@@ -1464,8 +1637,8 @@ create_page_one (GSLockPlug *plug)
 
         /* Status text */
 
-        plug->priv->auth_status_label = gtk_label_new (NULL);
-        gtk_box_pack_start (GTK_BOX (vbox), plug->priv->auth_status_label,
+        plug->priv->auth_message_label = gtk_label_new (NULL);
+        gtk_box_pack_start (GTK_BOX (vbox), plug->priv->auth_message_label,
                             FALSE, FALSE, 0);
         /* Buttons */
         plug->priv->auth_action_area = gtk_hbutton_box_new ();
@@ -1479,7 +1652,7 @@ create_page_one (GSLockPlug *plug)
 
         create_page_one_buttons (plug);
 
-        profile_end ("end", "page one");
+        gs_profile_end ("page one");
 }
 
 static void
@@ -1519,7 +1692,7 @@ create_page_two (GSLockPlug *plug)
         GtkWidget            *userlist_scroller;
         GtkWidget            *vbox;
 
-        profile_start ("start", "page two");
+        gs_profile_start ("page two");
 
         vbox = gtk_vbox_new (FALSE, 6);
         gtk_notebook_append_page (GTK_NOTEBOOK (plug->priv->notebook), vbox, NULL);
@@ -1556,7 +1729,7 @@ create_page_two (GSLockPlug *plug)
 
         create_page_two_buttons (plug);
 
-        profile_end ("end", "page two");
+        gs_profile_end ("page two");
 }
 
 static void
@@ -1647,10 +1820,11 @@ load_theme (GSLockPlug *plug)
         plug->priv->auth_action_area = glade_xml_get_widget (xml, "auth-action-area");
         plug->priv->auth_realname_label = glade_xml_get_widget (xml, "auth-realname-label");
         plug->priv->auth_username_label = glade_xml_get_widget (xml, "auth-username-label");
-        plug->priv->auth_short_username_label = glade_xml_get_widget (xml, "auth-short-username-label");
-        plug->priv->auth_password_entry = glade_xml_get_widget (xml, "auth-password-entry");
+        plug->priv->auth_prompt_label = glade_xml_get_widget (xml, "auth-prompt-label");
+        plug->priv->auth_prompt_entry = glade_xml_get_widget (xml, "auth-prompt-entry");
+        plug->priv->auth_prompt_box = glade_xml_get_widget (xml, "auth-prompt-box");
         plug->priv->auth_capslock_label = glade_xml_get_widget (xml, "auth-capslock-label");
-        plug->priv->auth_status_label = glade_xml_get_widget (xml, "auth-status-label");
+        plug->priv->auth_message_label = glade_xml_get_widget (xml, "auth-status-label");
         plug->priv->auth_unlock_button = glade_xml_get_widget (xml, "auth-unlock-button");
         plug->priv->auth_cancel_button = glade_xml_get_widget (xml, "auth-cancel-button");
         plug->priv->auth_logout_button = glade_xml_get_widget (xml, "auth-logout-button");
@@ -1685,13 +1859,14 @@ gs_lock_plug_init (GSLockPlug *plug)
 {
         gunichar              invisible_char;
 
-        profile_start ("start", NULL);
+        gs_profile_start (NULL);
 
         plug->priv = GS_LOCK_PLUG_GET_PRIVATE (plug);
 
         plug->priv->fusa_manager = NULL;
 
         if (! load_theme (plug)) {
+                gs_debug ("Unable to load theme!");
 
                 plug->priv->vbox = gtk_vbox_new (FALSE, 0);
 
@@ -1718,39 +1893,11 @@ gs_lock_plug_init (GSLockPlug *plug)
         gtk_widget_grab_default (plug->priv->auth_unlock_button);
 
         if (plug->priv->auth_username_label != NULL) {
-                char       *str;
-                char       *name;
-
-                /* FIXME should let the theme determine the font size */
-                name = get_user_on_host_name ();
-                str = g_strdup_printf ("<span size=\"small\">%s</span>", name);
-                g_free (name);
-                gtk_label_set_markup (GTK_LABEL (plug->priv->auth_username_label), str);
-                g_free (str);
-        }
-
-        if (plug->priv->auth_short_username_label != NULL) {
-                char       *str;
-                char       *name;
-
-                /* FIXME should let the theme determine the font size */
-                name = get_user_name ();
-                str = g_strdup_printf ("<span size=\"x-large\">%s</span>", name);
-                g_free (name);
-                gtk_label_set_markup (GTK_LABEL (plug->priv->auth_short_username_label), str);
-                g_free (str);
+                expand_string_for_label (plug->priv->auth_username_label);
         }
 
         if (plug->priv->auth_realname_label != NULL) {
-                char       *str;
-                char       *name;
-
-                /* FIXME should let the theme determine the font size */
-                name = get_user_display_name ();
-                str = g_strdup_printf ("<span size=\"x-large\">%s</span>", name);
-                g_free (name);
-                gtk_label_set_markup (GTK_LABEL (plug->priv->auth_realname_label), str);
-                g_free (str);
+                expand_string_for_label (plug->priv->auth_realname_label);
         }
 
         if (! plug->priv->logout_enabled
@@ -1768,15 +1915,15 @@ gs_lock_plug_init (GSLockPlug *plug)
                           G_CALLBACK (entry_key_press), plug);
 
         /* button press handler used to inhibit popup menu */
-        g_signal_connect (plug->priv->auth_password_entry, "button_press_event",
+        g_signal_connect (plug->priv->auth_prompt_entry, "button_press_event",
                           G_CALLBACK (entry_button_press), NULL);
-        gtk_entry_set_activates_default (GTK_ENTRY (plug->priv->auth_password_entry), TRUE);
-        gtk_entry_set_visibility (GTK_ENTRY (plug->priv->auth_password_entry), FALSE);
+        gtk_entry_set_activates_default (GTK_ENTRY (plug->priv->auth_prompt_entry), TRUE);
+        gtk_entry_set_visibility (GTK_ENTRY (plug->priv->auth_prompt_entry), FALSE);
 
         /* Only change the invisible character if it '*' otherwise assume it is OK */
-        if ('*' == gtk_entry_get_invisible_char (GTK_ENTRY (plug->priv->auth_password_entry))) {
+        if ('*' == gtk_entry_get_invisible_char (GTK_ENTRY (plug->priv->auth_prompt_entry))) {
                 invisible_char = INVISIBLE_CHAR_BLACK_CIRCLE;
-                gtk_entry_set_invisible_char (GTK_ENTRY (plug->priv->auth_password_entry), invisible_char);
+                gtk_entry_set_invisible_char (GTK_ENTRY (plug->priv->auth_prompt_entry), invisible_char);
         }
 
         g_signal_connect (plug->priv->auth_unlock_button, "clicked",
@@ -1812,7 +1959,7 @@ gs_lock_plug_init (GSLockPlug *plug)
 
         g_idle_add ((GSourceFunc)setup_treeview_idle, plug);
 
-        profile_end ("end", NULL);
+        gs_profile_end (NULL);
 }
 
 static void
@@ -1833,12 +1980,10 @@ gs_lock_plug_finalize (GObject *object)
                 g_object_unref (plug->priv->fusa_manager);
         }
 
-        remove_password_check_idle (plug);
-        remove_password_reset_idle (plug);
         remove_response_idle (plug);
         remove_monitor_idle (plug);
 
-        G_OBJECT_CLASS (parent_class)->finalize (object);
+        G_OBJECT_CLASS (gs_lock_plug_parent_class)->finalize (object);
 }
 
 GtkWidget *
