@@ -140,6 +140,71 @@ select_theme (GSManager *manager)
         return theme;
 }
 
+
+static GSJob *
+lookup_job_for_window (GSManager *manager,
+                       GSWindow  *window)
+{
+        GSJob *job;
+
+        if (manager->priv->jobs == NULL) {
+                return NULL;
+        }
+
+        job = g_hash_table_lookup (manager->priv->jobs, window);
+
+        return job;
+}
+
+static void
+manager_maybe_stop_job_for_window (GSManager *manager,
+                                   GSWindow  *window)
+{
+        GSJob *job;
+
+        job = lookup_job_for_window (manager, window);
+
+        if (job == NULL) {
+                gs_debug ("Job not found for window");
+                return;
+        }
+
+        gs_job_stop (job);
+}
+
+static void
+manager_maybe_start_job_for_window (GSManager *manager,
+                                    GSWindow  *window)
+{
+        GSJob *job;
+
+        job = lookup_job_for_window (manager, window);
+
+        if (job == NULL) {
+                gs_debug ("Job not found for window");
+                return;
+        }
+
+        if (! manager->priv->dialog_up) {
+                if (! manager->priv->throttled) {
+                        if (! gs_job_is_running (job)) {
+                                if (! gs_window_is_obscured (window)) {
+                                        gs_debug ("Starting job for window");
+                                        gs_job_start (job);
+                                } else {
+                                        gs_debug ("Window is obscured deferring start of job");
+                                }
+                        } else {
+                                gs_debug ("Not starting job because job is running");
+                        }
+                } else {
+                        gs_debug ("Not starting job because throttled");
+                }
+        } else {
+                gs_debug ("Not starting job because dialog is up");
+        }
+}
+
 static void
 cycle_job (GSWindow  *window,
            GSJob     *job,
@@ -151,7 +216,7 @@ cycle_job (GSWindow  *window,
 
         gs_job_stop (job);
         gs_job_set_theme (job, theme, NULL);
-        gs_job_start (job);
+        manager_maybe_start_job_for_window (manager, window);
 }
 
 static void
@@ -170,7 +235,7 @@ throttle_job (GSWindow  *window,
         if (manager->priv->throttled) {
                 gs_job_stop (job);
         } else {
-                gs_job_start (job);
+                manager_maybe_start_job_for_window (manager, window);
         }
 }
 
@@ -190,7 +255,7 @@ resume_job (GSWindow  *window,
         if (gs_job_is_running (job)) {
                 gs_job_suspend (job, FALSE);
         } else {
-                gs_job_start (job);
+                manager_maybe_start_job_for_window (manager, window);
         }
 }
 
@@ -452,6 +517,8 @@ gs_manager_cycle (GSManager *manager)
         g_return_val_if_fail (manager != NULL, FALSE);
         g_return_val_if_fail (GS_IS_MANAGER (manager), FALSE);
 
+        gs_debug ("cycling jobs");
+
         if (! manager->priv->active) {
                 return FALSE;
         }
@@ -837,6 +904,12 @@ window_dialog_up_cb (GSWindow  *window,
         g_signal_emit (manager, signals [AUTH_REQUEST_BEGIN], 0);
 
         manager->priv->dialog_up = TRUE;
+        /* Make all other windows insensitive so we don't get events */
+        for (l = manager->priv->windows; l; l = l->next) {
+                if (l->data != window) {
+                        gtk_widget_set_sensitive (GTK_WIDGET (l->data), FALSE);
+                }
+        }
 
         /* Move keyboard and mouse grabs so dialog can be used */
         gs_grab_move_to_window (manager->priv->grab,
@@ -848,13 +921,6 @@ window_dialog_up_cb (GSWindow  *window,
            the dialog can be used.  We'll regrab it when the dialog goes down. */
         gs_grab_release_mouse (manager->priv->grab);
 
-        /* Make all other windows insensitive so we don't get events */
-        for (l = manager->priv->windows; l; l = l->next) {
-                if (l->data != window) {
-                        gtk_widget_set_sensitive (GTK_WIDGET (l->data), FALSE);
-                }
-        }
-
         if (! manager->priv->throttled) {
                 gs_debug ("Suspending jobs");
 
@@ -882,40 +948,50 @@ window_dialog_down_cb (GSWindow  *window,
                 gtk_widget_set_sensitive (GTK_WIDGET (l->data), TRUE);
         }
 
+        manager->priv->dialog_up = FALSE;
+
         if (! manager->priv->throttled) {
                 manager_resume_jobs (manager);
         }
 
-        manager->priv->dialog_up = FALSE;
-
         g_signal_emit (manager, signals [AUTH_REQUEST_END], 0);
 }
 
-static void
-manager_maybe_start_job_for_window (GSManager *manager,
-                                    GSWindow  *window)
+static GSWindow *
+find_window_at_pointer (GSManager *manager)
 {
-        GSJob *job;
+        GdkDisplay *display;
+        GdkScreen  *screen;
+        int         monitor;
+        int         x, y;
+        GSWindow   *window;
+        int         screen_num;
+        GSList     *l;
 
-        if (manager->priv->jobs == NULL) {
-                return;
-        }
+        display = gdk_display_get_default ();
+        gdk_display_get_pointer (display, &screen, &x, &y, NULL);
+        monitor = gdk_screen_get_monitor_at_point (screen, x, y);
+        screen_num = gdk_screen_get_number (screen);
 
-        job = g_hash_table_lookup (manager->priv->jobs, window);
-
-        if (job == NULL) {
-                gs_debug ("Job not found for window");
-                return;
-        }
-
-        if (! manager->priv->dialog_up) {
-                if (! manager->priv->throttled) {
-                        if (! gs_job_is_running (job)) {
-                                gs_debug ("Starting job for window");
-                                gs_job_start (job);
-                        }
+        /* Find the gs-window that is on that screen */
+        window = NULL;
+        for (l = manager->priv->windows; l; l = l->next) {
+                GSWindow *win = GS_WINDOW (l->data);
+                if (gs_window_get_screen (win) == screen
+                    && gs_window_get_monitor (win) == monitor) {
+                        window = win;
                 }
         }
+
+        if (window == NULL) {
+                gs_debug ("WARNING: Could not find the GSWindow for screen %d", screen_num);
+                /* take the first one */
+                window = manager->priv->windows->data;
+        } else {
+                gs_debug ("Requesting unlock for screen %d", screen_num);
+        }
+
+        return window;
 }
 
 static gboolean
@@ -1051,16 +1127,58 @@ window_show_cb (GSWindow  *window,
         manager_show_window (manager, window);
 }
 
+static void
+maybe_set_window_throttle (GSManager *manager,
+                           GSWindow  *window,
+                           gboolean   throttled)
+{
+        if (throttled) {
+                manager_maybe_stop_job_for_window (manager, window);
+        } else {
+                manager_maybe_start_job_for_window (manager, window);
+        }
+}
+
+static void
+window_obscured_cb (GSWindow   *window,
+                    GParamSpec *pspec,
+                    GSManager  *manager)
+{
+        gboolean obscured;
+
+        obscured = gs_window_is_obscured (window);
+        gs_debug ("Handling window obscured: %s", obscured ? "obscured" : "unobscured");
+
+        maybe_set_window_throttle (manager, window, obscured);
+
+        if (! obscured) {
+                gs_manager_request_unlock (manager);
+        }
+}
+
+static gboolean
+window_activity_cb (GSWindow  *window,
+                    GSManager *manager)
+{
+        gboolean handled;
+
+        handled = gs_manager_request_unlock (manager);
+
+        return handled;
+}
+
 static void
 disconnect_window_signals (GSManager *manager,
                            GSWindow  *window)
 {
         g_signal_handlers_disconnect_by_func (window, window_deactivated_cb, manager);
+        g_signal_handlers_disconnect_by_func (window, window_activity_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_dialog_up_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_dialog_down_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_show_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_map_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_map_event_cb, manager);
+        g_signal_handlers_disconnect_by_func (window, window_obscured_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_unmap_cb, manager);
         g_signal_handlers_disconnect_by_func (window, window_grab_broken_cb, manager);
 }
@@ -1078,6 +1196,8 @@ connect_window_signals (GSManager *manager,
 {
         g_signal_connect_object (window, "destroy",
                                  G_CALLBACK (window_destroyed_cb), manager, 0);
+        g_signal_connect_object (window, "activity",
+                                 G_CALLBACK (window_activity_cb), manager, 0);
         g_signal_connect_object (window, "deactivated",
                                  G_CALLBACK (window_deactivated_cb), manager, 0);
         g_signal_connect_object (window, "dialog-up",
@@ -1090,6 +1210,8 @@ connect_window_signals (GSManager *manager,
                                  G_CALLBACK (window_map_cb), manager, G_CONNECT_AFTER);
         g_signal_connect_object (window, "map_event",
                                  G_CALLBACK (window_map_event_cb), manager, G_CONNECT_AFTER);
+        g_signal_connect_object (window, "notify::obscured",
+                                 G_CALLBACK (window_obscured_cb), manager, G_CONNECT_AFTER);
         g_signal_connect_object (window, "unmap",
                                  G_CALLBACK (window_unmap_cb), manager, G_CONNECT_AFTER);
         g_signal_connect_object (window, "grab_broken_event",
@@ -1301,20 +1423,22 @@ gs_manager_get_active (GSManager *manager)
         return manager->priv->active;
 }
 
-void
+gboolean
 gs_manager_request_unlock (GSManager *manager)
 {
-        GSList     *l;
-        GdkDisplay *display;
-        GdkScreen  *screen;
-        int         screen_num;
+        GSWindow *window;
 
-        g_return_if_fail (manager != NULL);
-        g_return_if_fail (GS_IS_MANAGER (manager));
+        g_return_val_if_fail (manager != NULL, FALSE);
+        g_return_val_if_fail (GS_IS_MANAGER (manager), FALSE);
 
         if (! manager->priv->active) {
                 gs_debug ("Request unlock but manager is not active");
-                return;
+                return FALSE;
+        }
+
+        if (manager->priv->dialog_up) {
+                gs_debug ("Request unlock but dialog is already up");
+                return FALSE;
         }
 
         if (manager->priv->fading) {
@@ -1322,19 +1446,14 @@ gs_manager_request_unlock (GSManager *manager)
                 gs_fade_finish (manager->priv->fade);
         }
 
-        /* Find the screen that contains the pointer */
-        display = gdk_display_get_default ();
-        gdk_display_get_pointer (display, &screen, NULL, NULL, NULL);
-        screen_num = gdk_screen_get_number (screen);
-
-        /* Find the gs-window that is on that screen */
-        for (l = manager->priv->windows; l; l = l->next) {
-                int num;
-
-                num = gdk_screen_get_number (GTK_WINDOW (l->data)->screen);
-                if (num == screen_num) {
-                        gs_window_request_unlock (GS_WINDOW (l->data));
-                        break;
-                }
+        if (manager->priv->windows == NULL) {
+                gs_debug ("We don't have any windows!");
+                return FALSE;
         }
+
+        /* Find the GSWindow that contains the pointer */
+        window = find_window_at_pointer (manager);
+        gs_window_request_unlock (window);
+
+        return TRUE;
 }
