@@ -63,8 +63,10 @@ gboolean    gs_manager_get_active           (GSManager  *manager);
 
 gboolean    gs_manager_cycle                (GSManager  *manager);
 
+void        gs_manager_get_lock_active      (GSManager  *manager,
+                                             gboolean   *lock_active);
 void        gs_manager_set_lock_active      (GSManager  *manager,
-                                             gboolean    lock_enabled);
+                                             gboolean    lock_active);
 void        gs_manager_set_lock_enabled     (GSManager  *manager,
                                              gboolean    lock_enabled);
 void        gs_manager_set_lock_timeout     (GSManager  *manager,
@@ -85,7 +87,7 @@ void        gs_manager_set_themes           (GSManager  *manager,
                                              GSList     *themes);
 void        gs_manager_set_mode             (GSManager  *manager,
                                              GSSaverMode mode);
-void        gs_manager_request_unlock       (GSManager  *manager);
+gboolean    gs_manager_request_unlock       (GSManager  *manager);
 
 G_END_DECLS
 
