@@ -1,6 +1,6 @@
 /* -*- Mode: C; tab-width: 8; indent-tabs-mode: nil; c-basic-offset: 8 -*-
  *
- * Copyright (C) 2004-2005 William Jon McCann <mccann@jhu.edu>
+ * Copyright (C) 2004-2006 William Jon McCann <mccann@jhu.edu>
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -46,18 +46,10 @@ typedef struct
 
         GSPrefsPrivate  *priv;
 
-        gboolean         debug;
-        gboolean         verbose;		/* whether to print out lots of status info */
-
-        gboolean         idle_activation_enabled; /* whether to activate when idle */
-        gboolean         lock_enabled;		/* whether to lock when active */
-        gboolean         logout_enabled;	/* Whether to offer the logout option */
-        gboolean         user_switch_enabled;	/* Whether to offer the user switch option */
-
-        gboolean         dpms_enabled;		/* Whether power management is enabled */
-        guint            dpms_standby;		/* how long until monitor goes black */
-        guint            dpms_suspend;		/* how long until monitor power-saves */
-        guint            dpms_off;		/* how long until monitor powers down */
+        guint            idle_activation_enabled : 1; /* whether to activate when idle */
+        guint            lock_enabled : 1;		/* whether to lock when active */
+        guint            logout_enabled : 1;	/* Whether to offer the logout option */
+        guint            user_switch_enabled : 1;	/* Whether to offer the user switch option */
 
         guint            timeout;	       	/* how much idle time before activation */
         guint            lock_timeout;		/* how long after activation locking starts */
